class Actions(object):
    LIST = 'LIST'
    RUN = 'RUN'
    RESTART = 'RESTART'
    STOP = 'STOP'
    REMOVE = 'REMOVE'
    SSH = 'SSH'


def parseargs(argv):
    import argparse

    class ExceptionHandledArgumentParser(argparse.ArgumentParser):
        def error(self, message):
            raise argparse.ArgumentError(None, message)

    from .remotedocker import __version__
    parser = ExceptionHandledArgumentParser(
        'RemoteDocker (version {}) : Run your script in a docker on another machine as if it were on yours'.format(
            __version__))
    subparse = parser.add_subparsers()

    parse_list = subparse.add_parser('list')
    parse_list.set_defaults(action=Actions.LIST)

    parse_run = subparse.add_parser('run')
    parse_run.add_argument('command', nargs=argparse.REMAINDER)
    parse_run.add_argument('--tag', help='a tag, latest used if not provided (only for running the old tag)')
    parse_run.add_argument('--host', help='host')
    parse_run.add_argument('--path', help='remote path')
    parse_run.add_argument('--docker',
                           help='docker executable, you can provide this value to be something like `nvidia-docker` (default: docker)')
    parse_run.set_defaults(action=Actions.RUN)

    parse_restart = subparse.add_parser('restart')
    parse_restart.add_argument('tag', nargs='?', help='a tag, latest used if not provided')
    parse_restart.set_defaults(action=Actions.RESTART)

    parse_stop = subparse.add_parser('stop')
    parse_stop.add_argument('tag', nargs='?', help='a tag, latest used if not provided')
    parse_stop.set_defaults(action=Actions.STOP)

    parse_remove = subparse.add_parser('rm')
    parse_remove.add_argument('tag', help='a tag, you must specify this explicitly')
    parse_remove.set_defaults(action=Actions.REMOVE)

    parse_ssh = subparse.add_parser('ssh')
    parse_ssh.add_argument('tag', nargs='?', help='a tag of which host we will ssh into')
    parse_ssh.set_defaults(action=Actions.SSH)

    try:
        args = parser.parse_args(argv)

        if not getattr(args, 'action', None):
            raise argparse.ArgumentError(None, 'No action provided')

        return args
    except argparse.ArgumentError as e:
        from src import utils
        raise utils.errors.ArgumentError(str(e))
